"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RunningProcess = void 0;
const childProcess = require("child_process");
const mergeStream = require("merge-stream");
const util = require("util");
const scanner = require("./scanner");
const delay = util.promisify(setTimeout);
/** a long-running process whose behavior can be observed at runtime */
class RunningProcess {
    constructor(args) {
        this.endedCallbacks = [];
        this.childProcess = childProcess.spawn(args.runnable, args.params, {
            cwd: args.cwd,
            env: args.env,
        });
        this.childProcess.on("close", this.onClose.bind(this));
        if (this.childProcess.stdin == null) {
            throw new Error("process.stdin should not be null"); // this exists only to make the typechecker shut up
        }
        this.stdin = this.childProcess.stdin;
        if (this.childProcess.stdout == null) {
            throw new Error("process.stdout should not be null"); // NOTE: this exists only to make the typechecker shut up
        }
        this.stdout = scanner.wrapStream(this.childProcess.stdout);
        if (this.childProcess.stderr == null) {
            throw new Error("process.stderr should not be null"); // NOTE: this exists only to make the typechecker shut up
        }
        this.stderr = scanner.wrapStream(this.childProcess.stderr);
        const outputStream = mergeStream(this.childProcess.stdout, this.childProcess.stderr);
        this.output = scanner.wrapStream(outputStream);
    }
    /** stops the currently running process */
    kill() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.result) {
                throw new Error("process has already finished and cannot be killed anymore");
            }
            this.result = {
                state: "killed",
                stdText: this.stdout.fullText(),
                errText: this.stderr.fullText(),
                combinedText: this.output.fullText(),
            };
            this.childProcess.kill();
            yield delay(1);
            return this.result;
        });
    }
    /** returns the process ID of the underlying ChildProcess */
    pid() {
        return this.childProcess.pid;
    }
    /** returns a promise that resolves when the underlying ChildProcess terminates */
    waitForEnd() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.result) {
                return this.result;
            }
            return new Promise((resolve) => {
                this.endedCallbacks.push(resolve);
            });
        });
    }
    /**
     * Called when the underlying ChildProcess ends.
     * This gets called for natural ends and kills.
     */
    onClose(exitCode) {
        if (exitCode != null) {
            const finishedProcess = {
                exitCode,
                errText: this.stderr.fullText(),
                combinedText: this.output.fullText(),
                state: "finished",
                stdText: this.stdout.fullText(),
            };
            this.result = finishedProcess;
        }
        else {
            const killedProcess = {
                errText: this.stderr.fullText(),
                combinedText: this.output.fullText(),
                state: "killed",
                stdText: this.stdout.fullText(),
            };
            this.result = killedProcess;
        }
        for (const endedCallback of this.endedCallbacks) {
            endedCallback(this.result);
        }
    }
}
exports.RunningProcess = RunningProcess;
//# sourceMappingURL=running-process.js.map